// ListCtrl.cpp

#include <afxwin.h>
#include <afxcmn.h>
#include <windowsx.h>
#include "ListCtrl.h"
#include <tchar.h>


void ListCtrl::Initialize ( void )
{
    ASSERT_VALID( this );
    
    // Put any common initialization here, e.g. style modifications:

#ifdef LVS_EX_FULLROWSELECT
  ListView_SetExtendedListViewStyle( m_hWnd, LVS_EX_FULLROWSELECT );
#endif
    
    DoInitialize();
}


void ListCtrl::DoDestroy( void ) 
{
}


int CALLBACK ListCtrl::CompareProc(
    LPARAM lParam1, LPARAM lParam2, LPARAM lParam )
{
    ListCtrl *pThis = reinterpret_cast< ListCtrl * >( lParam );
    ASSERT_POINTER( pThis, ListCtrl );
    ASSERT_VALID( pThis );
    
    int nRet = 0;
    
    const int nColumns = pThis->GetColumnCount();
    for ( int iColumn = 0; iColumn < nColumns; ++iColumn ) {
        const int nSortColumn = pThis->GetSubColumn( iColumn );
        
        const CString str1 = pThis->DoGetText( 
            lParam1, nSortColumn, /* bSorting = */ TRUE );
        const CString str2 = pThis->DoGetText( 
            lParam2, nSortColumn, /* bSorting = */ TRUE );
        
        ASSERT( AfxIsValidString( str1 ) );
        ASSERT( AfxIsValidString( str2 ) );
        
        nRet = str1.Collate( str2 );
        if ( 0 != nRet ) {
            break;
        }
    }
    
    if ( !pThis->m_bAscending ) {
        nRet = -nRet;
    }
    
    return nRet;
}


void ListCtrl::SetSortColumn( int nSortColumn )
{
    ASSERT_VALID( this );
    
    CWaitCursor wc;
    
    if ( m_nSortColumn != nSortColumn ) {
        m_bAscending = TRUE;
    } else {
        m_bAscending = !m_bAscending;
    }
    m_nSortColumn = nSortColumn;

    Sort();
}


BOOL ListCtrl::OnCommand( WPARAM wParam, LPARAM ) 
{
    ASSERT_VALID( this );
    SetSortColumn( GET_WM_COMMAND_ID( wParam, lParam ) );	
    return TRUE;
}


void ListCtrl::OnColumnclick( NMHDR* pNMHDR, LRESULT* pResult )
{
    ASSERT_VALID( this );
    
    const NM_LISTVIEW* pNMListView = 
        reinterpret_cast< NM_LISTVIEW * >( pNMHDR );
    ASSERT_POINTER( pNMHDR, NM_LISTVIEW );
    SetSortColumn( pNMListView->iSubItem );
    
    ASSERT_POINTER( pResult, LRESULT );
    *pResult = 0;
}


void ListCtrl::OnGetdispinfo( NMHDR *pNMHDR, LRESULT *pResult ) 
{
    ASSERT_VALID( this );
    
    LV_DISPINFO *pDispInfo = 
        reinterpret_cast< LV_DISPINFO * >( pNMHDR );
    ASSERT_POINTER( pDispInfo, LV_DISPINFO );
    
    // This implementation does not handle images; must be text:
    ASSERT( LVIF_TEXT == pDispInfo->item.mask );
    
    // Do NOT initialize str directly from GetText, like this:
    //   static CString str = DoGetText( ... ), 
    // Static variables are initialized only the first time through.
    // Note that str MUST be static to survive function's end...
    static CString str; 
    str = DoGetText( GetItemData( pDispInfo->item.iItem ), 
        pDispInfo->item.iSubItem );
    pDispInfo->item.pszText = (LPTSTR)(LPCTSTR) str;
    
    ASSERT_POINTER( pResult, LRESULT );
    *pResult = 0;
}


void ListCtrl::OnContextMenu( CWnd *, CPoint point )
{
    ASSERT_VALID(this);
    
    const int nColumns = GetColumnCount();
    const CHeaderCtrl *pwndHeader = 
        static_cast< CHeaderCtrl * >( GetWindow( GW_CHILD ) );
    ASSERT_VALID( pwndHeader );
    
    // Adjust position: point is -1, -1 if triggered by Shift+F10:
    if ( -1 == point.x && -1 == point.y ) {
        CRect rcList, rcHeaderClient;
        pwndHeader->GetClientRect( &rcHeaderClient );
        const int nOffset = rcHeaderClient.Height();
        GetWindowRect( &rcList );
        point = rcList.TopLeft() + CSize( nOffset, 2 * nOffset );
    }
    
    // Create the popup menu if we haven't already:
    if ( 0 == m_pmenuPopup ) {
        m_pmenuPopup = new CMenu();
        ASSERT( 0 != m_pmenuPopup );
        VERIFY( m_pmenuPopup->CreatePopupMenu() );
        
        // Add actual column names to menu:
        for ( int iColumn = 0; iColumn < nColumns; ++iColumn ) {
            TCHAR szTitle[ 32 ];
            HDITEM hi = { HDI_TEXT, 0, szTitle, 0, sizeof szTitle };
            pwndHeader->GetItem( iColumn, &hi );
            m_pmenuPopup->AppendMenu ( 
                MF_BYCOMMAND | MF_STRING, iColumn, szTitle );
        }
    }
    
    m_pmenuPopup->CheckMenuRadioItem(
        0, nColumns, m_nSortColumn, MF_BYCOMMAND );
    m_pmenuPopup->TrackPopupMenu( 
        TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this );
}


void ListCtrl::OnDestroy() 
{
    DoDestroy();
    if ( 0 != m_pmenuPopup ) {
        m_pmenuPopup->DestroyMenu();
        delete m_pmenuPopup;
    }
}


BEGIN_MESSAGE_MAP( ListCtrl, CListCtrl )

ON_NOTIFY_REFLECT( LVN_COLUMNCLICK, OnColumnclick )
ON_NOTIFY_REFLECT( LVN_GETDISPINFO, OnGetdispinfo )
ON_WM_CONTEXTMENU()
ON_WM_DESTROY()

END_MESSAGE_MAP()

// end of file
